'''
Created on Feb 21, 2014

@author: mmi46
'''
import numpy as np


def getTeff(Z_Haz, R_Haz, N_Haz, Soil_type, Disp_eff, **kwargs):
    '''
    Returns the effective period of the structure
     using the values from NZS1170.5
    :param BO:
    :param Disp_eff:
    '''
    gravity = kwargs.get('gravity', 9.8)
    if Soil_type == 'C':
        T_C = 3.0
        D_C = (3.96 * Z_Haz * R_Haz *
               N_Haz / (2 * np.pi) ** 2 * gravity)
    elif Soil_type == 'D':
        T_C = 3.0
        D_C = (6.42 * Z_Haz * R_Haz *
               N_Haz / (2 * np.pi) ** 2 * gravity)
    elif Soil_type == 'E':
        T_C = 3.0
        D_C = (9.96 * Z_Haz * R_Haz * \
               N_Haz / (2 * np.pi) ** 2 * gravity)

    if Disp_eff > D_C:
        'Print displacement ductility not achieveable'
        T_eff = 0
    else:
        T_eff = T_C * Disp_eff / D_C

    return T_eff


def calculate_z(corner_disp, site_class_nzs):
    '''
    calculates the value of Z for design for a given corner_disp.
    NOTE: corner_disp must be at 3s
    '''
    if site_class_nzs == 'C':
        i = 0
    elif site_class_nzs == 'D':
        i = 1
    elif site_class_nzs == 'E':
        i = 2
    else:
        raise KeyError("site_class_nzs must be 'C', 'D' or 'E'.")

    nzs_val = [3.96, 6.42, 9.96]
    Z = corner_disp * 4 * np.pi ** 2 / nzs_val[i] / 9.8
    return Z


sae = {
    "A": {
        "S": 1.0,
        "TB": 0.15,
        "TC": 0.4,
        "TD": 2.0
    },
    "B": {
        "S": 1.2,
        "TB": 0.15,
        "TC": 0.5,
        "TD": 2.0
    },
    "C": {
        "S": 1.15,
        "TB": 0.2,
        "TC": 0.6,
        "TD": 2.0
    },
    "D": {
        "S": 1.35,
        "TB": 0.2,
        "TC": 0.8,
        "TD": 2.0
    },
    "E": {
        "S": 1.4,
        "TB": 0.15,
        "TC": 0.5,
        "TD": 2.0
    }
}


def eurocode_sa(t, sc):
    """
    Eurocode site response spectrum Part 1 CL 3.2.2.2
    :param t:
    :param sc:
    :return:
    """
    eta = 1.0  # for 5% damping CL 3.2.2.2
    if 0 < t <= sae[sc]["TB"]:
        sa = sae[sc]["S"] * (1 + t / sae[sc]['TB'] * eta * 2.5 - 1)
    elif sae[sc]["TB"] < t <= sae[sc]["TC"]:
        sa = sae[sc]["S"] * eta * 2.5
    elif sae[sc]["TC"] < t <= sae[sc]["TD"]:
        sa = sae[sc]["S"] * eta * 2.5 * sae[sc]['TC'] / t
    elif sae[sc]["TD"] < t <= 4.0:
        sa = sae[sc]["S"] * eta * 2.5 * (sae[sc]['TC'] * sae[sc]['TD']) / t ** 2
    else:
        # beyond the scope of the standard
        raise NotImplementedError
    return sa



import numpy as np

from eqdes.extensions.exceptions import DesignError


def assess(fb, storey_forces, mom_ratio=0.6, verbose=0):
    """
    Distribute the applied loads to a frame structure
    :param fb: FrameBuilding object
    :param mom_ratio: ratio of overturning moment that is resisted by column base hinges
    :param verbose: level of verbosity
    :return: beam moments, column base moments, seismic axial loads in exterior columns
    """
    if hasattr(fb, 'column_depth') and np.std(fb.column_depth) > 1e-2:
        print('Does not work with odd column depths')
        print(fb.column_depth)
        raise NotImplementedError

    height = np.zeros(len(fb.storey_heights))
    for i in range(len(fb.storey_heights)):
        if i == 0:
            height[i] = fb.storey_heights[i]
        else:
            height[i] = height[i - 1] + fb.storey_heights[i]

    mom_running = 0
    mom_storey = np.zeros(len(fb.storey_heights))
    v_storey = np.zeros(len(fb.storey_heights))
    for i in range(len(fb.storey_heights)):

        if i == 0:
            v_storey[-1 - i] = storey_forces[-1 - i]
        else:
            v_storey[-1 - i] = v_storey[-i] + storey_forces[-1 - i]
        mom_storey[-1 - i] = (v_storey[-1 - i] *
            fb.storey_heights[-1 - i] + mom_running)
        mom_running = mom_storey[-1 - i]

    cumulative_total_shear = sum(v_storey)
    base_shear = sum(storey_forces)

    # Column_base_moment_total=mom_storey[0]*Base_moment_contribution
    column_base_moment_total = base_shear * mom_ratio * fb.storey_heights[0]
    moment_column_bases = (column_base_moment_total / fb.n_bays * np.ones((fb.n_bays + 1)))
    moment_column_bases[0] = moment_column_bases[0] / 2
    moment_column_bases[-1] = moment_column_bases[-1] / 2

    axial_seismic = (mom_storey[0] - column_base_moment_total) / sum(fb.bay_lengths)
    if verbose == 1:
        print('Storey shear forces: \n', v_storey)
        print('Moments', mom_storey)
        print('Total overturning moment: ', mom_storey[0])
        print('column_base_moment_total: ', column_base_moment_total)
        print('Seismic axial: ', axial_seismic)

    beam_shear_force = np.zeros(fb.n_storeys)

    for i in range(int(np.ceil(fb.n_storeys / fb.beam_group_size))):
        group_shear = np.average(
            v_storey[i * fb.beam_group_size:(i + 1) * fb.beam_group_size]) / cumulative_total_shear * axial_seismic
        if verbose > 1:
            print('group shear: ', group_shear)
        for j in range(int(fb.beam_group_size)):
            if i * fb.beam_group_size + j == fb.n_storeys:
                if verbose:
                    print('odd number of storeys')
                break
            beam_shear_force[i * fb.beam_group_size + j] = group_shear

    if (sum(beam_shear_force) - axial_seismic) / axial_seismic > 1e-2:
        raise DesignError('Beam shear force incorrect!')

    moment_beams_cl = np.zeros((fb.n_storeys, fb.n_bays, 2))
    if fb.bay_lengths[0] != fb.bay_lengths[-1]:
        print('Design not developed for irregular frames!')
    else:
        for i in range(fb.n_storeys):
            for j in range(fb.n_bays):
                moment_beams_cl[i][j] = beam_shear_force[i] * fb.bay_lengths[0] * 0.5 * np.array([1, -1])

    if verbose > 0:
        print('Seismic beam shear force: \n', beam_shear_force)
        print('Beam centreline moments: \n', moment_beams_cl)
    
    return moment_beams_cl, moment_column_bases, axial_seismic






import numpy as np

from sfsimodels import loader as ml
from sfsimodels import output as mo

from eqdes import models as sm
from eqdes import ddbd_tools as dt
from eqdes import nonlinear_foundation as nf


def dba_frame(fb, hz, theta_max, otm_max, **kwargs):
    """
    Displacement-based assessment of a frame building
    :param fb: FrameBuilding Object
    :param hz: Hazard Object
    :param theta_max: [degrees], maximum structural interstorey drift
    :param otm_max: [N], maximum overturning moment
    :param kwargs:
    :return:
    """

    af = sm.AssessedFrame(fb, hz)
    af.otm_max = otm_max
    af.theta_max = theta_max
    verbose = kwargs.get('verbose', af.verbose)

    ductility_reduction_factors = 100
    theta_c = theta_max
    for i in range(ductility_reduction_factors):
        mu_reduction_factor = 1.0 - float(i) / ductility_reduction_factors
        theta_c = theta_max * mu_reduction_factor
        displacements = dt.displacement_profile_frame(theta_c, af.heights, af.hm_factor)
        af.delta_max, af.mass_eff, af.height_eff = dt.equivalent_sdof(af.storey_mass_p_frame, displacements, af.heights)
        af.theta_y = dt.conc_frame_yield_drift(af.fye, af.concrete.youngs_steel, af.av_bay, af.av_beam)
        af.delta_y = dt.yield_displacement(af.theta_y, af.height_eff)
        af.mu = dt.ductility(af.delta_max, af.delta_y)
        if i == 0:
            af.max_mu = af.mu
        af.xi = dt.equivalent_viscous_damping(af.mu)
        af.eta = dt.reduction_factor(af.xi)
        otm = otm_max * dt.bilinear_load_factor(af.mu, af.max_mu, af.post_yield_stiffness_ratio)
        af.v_base = otm / af.height_eff
        af.k_eff = af.v_base / af.delta_max
        af.t_eff = dt.effective_period_from_stiffness(af.mass_eff, af.k_eff)

        af.delta_demand = dt.displacement_from_effective_period(af.eta, af.hz.corner_disp,
                                                                af.t_eff, af.hz.corner_period)

        if verbose > 1:
            print('Delta_D: ', af.delta_max)
            print('Effective mass: ', af.mass_eff)
            print('Effective height: ', af.height_eff)
            print('Mu: ', af.mu)
            print('theta yield', af.theta_y)
            print('xi: ', af.xi)
            print('Reduction Factor: ', af.eta)
            print('t_eff', af.t_eff)
        if af.delta_demand > af.delta_max:  # failure occurs
            af.mu = af.delta_demand / af.delta_y
            # af.delta_demand
            break
        else:
            if verbose > 1:
                print("drift %.2f is not compatible" % theta_c)
    af.assessed_drift = theta_c
    return af


def frame_sfsi(fb, hz, sl, fd, theta_max, otm_max, found_rot=0.00001, **kwargs):
    """
    Displacement-based assessment of a frame building considering SFSI
    :param fb: FrameBuilding Object
    :param hz: Hazard Object
    :param theta_max: [degrees], maximum structural interstorey drift
    :param otm_max: [N], maximum overturning moment
    :param found_rot: [rad], initial guess of foundation rotation
    :param kwargs:
    :return:
    """

    af = sm.AssessedSFSIFrame(fb, hz, sl, fd)
    af.otm_max = otm_max
    af.theta_max = theta_max
    af.theta_f = found_rot
    verbose = kwargs.get('verbose', af.verbose)
    af.static_values()

    # add foundation to heights
    heights = list(af.heights)
    heights.insert(0, 0)
    heights = np.array(heights) + af.fd.height
    # add foundation to masses
    storey_masses = list(af.storey_masses)
    storey_masses.insert(0, af.fd.mass)
    storey_masses = np.array(storey_masses)
    af.storey_mass_p_frame = storey_masses / af.n_seismic_frames

    ductility_reduction_factors = 100
    iterations_ductility = kwargs.get('iterations_ductility', ductility_reduction_factors)
    iterations_rotation = kwargs.get('iterations_rotation', 20)
    theta_c = theta_max

    for i in range(iterations_ductility):
        mu_reduction_factor = 1.0 - float(i) / ductility_reduction_factors
        theta_c = theta_max * mu_reduction_factor
        displacements = dt.displacement_profile_frame(theta_c, heights, af.hm_factor, foundation=True,
                                                    fd_height=af.fd.height, theta_f=af.theta_f)
        af.delta_max, af.mass_eff, af.height_eff = dt.equivalent_sdof(af.storey_mass_p_frame, displacements, heights)
        af.theta_y = dt.conc_frame_yield_drift(af.fye, af.concrete.youngs_steel, af.av_bay, af.av_beam)
        af.delta_y = dt.yield_displacement(af.theta_y, af.height_eff - af.fd.height)
        approx_delta_f = af.theta_f * af.height_eff
        af.delta_ss = af.delta_max - approx_delta_f
        af.mu = dt.ductility(af.delta_ss, af.delta_y)
        if i == 0:
            af.max_mu = af.mu
        af.xi = dt.equivalent_viscous_damping(af.mu)
        eta_ss = dt.reduction_factor(af.xi)

        otm = otm_max * dt.bilinear_load_factor(af.mu, af.max_mu, af.post_yield_stiffness_ratio)

        # Foundation behaviour
        eta_fshear = nf.foundation_shear_reduction_factor()
        af.delta_fshear = af.v_base / (0.5 * af.k_f0_shear)
        moment_f = otm
        found_rot_tol = 0.00001
        bhr = (af.fd.width / af.height_eff)
        for j in range(iterations_rotation):
            norm_rot = found_rot / af.theta_pseudo_up
            cor_norm_rot = nf.calculate_corrected_normalised_rotation(norm_rot, bhr)
            if verbose > 1:
                print("soil_q: ", af.soil_q)
                print("axial load ratio: ", af.axial_load_ratio)
                print("theta_f: ", af.theta_f)
                print('cor_norm_rot: ', cor_norm_rot)
            stiffness_ratio = nf.foundation_rotation_stiffness_ratio(cor_norm_rot)
            k_f_eff = af.k_f_0 * stiffness_ratio
            temp_found_rot = found_rot

            found_rot = moment_f / k_f_eff

            iteration_diff = (abs(temp_found_rot - found_rot) / found_rot)
            if iteration_diff < found_rot_tol:
                break
        eta_frot = nf.foundation_rotation_reduction_factor(cor_norm_rot)
        af.delta_frot = af.theta_f * af.height_eff
        af.delta_f = af.delta_frot + af.delta_fshear
        af.delta_max = af.delta_ss + af.delta_f
        eta_sys = nf.system_reduction_factor(af.delta_ss, af.delta_frot, af.delta_fshear, eta_ss, eta_frot, eta_fshear)

        af.eta = eta_sys
        af.theta_f = found_rot
        af.v_base = otm / (af.height_eff - af.fd.height)  # Assume hinge at top of foundation.
        af.k_eff = af.v_base / af.delta_max
        af.t_eff = dt.effective_period_from_stiffness(af.mass_eff, af.k_eff)
        if verbose > 1:
            print('Delta_max: ', af.delta_max)
            print('Effective mass: ', af.mass_eff)
            print('Effective height: ', af.height_eff)
            print('Mu: ', af.mu)
            print('theta yield', af.theta_y)
            print('xi: ', af.xi)
            print('Reduction Factor: ', af.eta)
            print('t_eff', af.t_eff)

        af.delta_demand = dt.displacement_from_effective_period(af.eta, af.hz.corner_disp,
                                                                af.t_eff, af.hz.corner_period)

        if af.delta_demand > af.delta_max:  # failure occurs
            af.mu = (af.delta_demand - af.delta_f) / af.delta_y
            # af.delta_demand
            break
        else:
            if verbose > 1:
                print("drift %.2f is not compatible" % theta_c)
    af.assessed_drift = theta_c
    return af


def run_frame_dba_fixed():
    fb = sm.FrameBuilding()
    hz = sm.Hazard()
    hz = ml.load_hazard_sample_data(hz)
    ml.load_frame_building_sample_data(fb)

    designed_frame = dba_frame(fb, hz, theta_max=0.035, otm_max=3e5, verbose=0)



"""
References:
Sullivan, Salawdeh S., Pecker A. (2010). Soil-foundation-structure interaction considerations
 for performance-based design of RC wall structures on shallow foundations.
"""

import numpy as np

from eqdes.extensions.exceptions import DesignError


def displacement_profile_frame(theta_c, heights, hm_factor, foundation=False,
                         fd_height=0.0, theta_f=0.0, verbose=0):
    heights = np.array(heights)
    if foundation:
        displaced_shape = np.zeros(len(heights))
        heights_ss = heights[1:] - fd_height
        displaced_shape[1:] += cal_displaced_shape(theta_c, heights_ss, btype="frame")
        displaced_shape += theta_f * heights
    else:
        displaced_shape = cal_displaced_shape(theta_c, heights, btype="frame")
    displacements = displaced_shape * hm_factor
    if verbose:
        print('theta_c: ', theta_c)
        print('hm_factor: ', hm_factor)
        print('displacements: ', displacements)
    return displacements


def cal_higher_mode_factor(n_storeys, btype="frame"):
    """
    Calculates the higher mode factor according to DDBD12 CL.
    :param n_storeys:
    :param btype:
    :return:
    """
    # Higher mode factor for frame structures
    if btype == "frame":
        if n_storeys < 6:
            factor = 1.0
        elif n_storeys <= 16:
            factor = 1.0 - 0.15 * float((n_storeys - 6.0)) / (16.0 - 6.0)
        else:
            factor = 0.85
    else:
        if n_storeys < 10:
            factor = 1.0
        elif n_storeys <= 16:
            factor = (1.0 - 0.06 * float((n_storeys - 10.0)) / (16.0 - 10.0))
        else:
            factor = 0.85
    return factor


def cal_displaced_shape(theta_c, heights, btype="frame"):
    heights = np.array(heights)
    max_height = max(heights)
    if btype == "frame":
        return theta_c * heights * (4 * max_height - heights) / (4 * max_height - heights[0])


def equivalent_sdof(masses, displacements, heights):
    mass_x_disp = masses * displacements
    mass_x_disp2 = masses * displacements ** 2
    mass_x_disp_x_height = masses * displacements * heights

    delta_d = sum(mass_x_disp2) / sum(mass_x_disp)
    mass_eff = sum(mass_x_disp) / delta_d
    height_eff = sum(mass_x_disp_x_height) / sum(mass_x_disp)

    return delta_d, mass_eff, height_eff

    
def yield_displacement(theta_y, height_eff):  # TODO: check where this comes from?
    return theta_y * height_eff


def yield_displacement_wall(phi_y, heights, max_height):
    """
    The yield displacement of a concrete wall. Eq. 6 (Sullivan et al. 2010).
    :param phi_y: yield curvature
    :param heights: height
    :return:
    """

    return phi_y * heights ** 2 / 2 + phi_y * heights ** 3 / max_height


def conc_frame_yield_drift(fye, youngs_steel, av_bay_length, av_beam_depth):
    """
    Yield drift of a concrete frame from DDBD (Priestley et al. (2007)
    :param fye: Effective yeild strength of reinforcing steel
    :param youngs_steel: Young's modulus of reinforcing steel
    :param av_bay_length: Average bay length
    :param av_beam_depth: average beam depth
    :return:
    """
    return 0.5 * fye / youngs_steel * (av_bay_length / av_beam_depth)


def ductility(delta_current, delta_y):
    """
    Computes the ductility for a given displacement and yield.
    :param delta_current: current displacement
    :param delta_y: yield displacement
    :return:
    """
    return delta_current / delta_y


def reduction_factor(xi, near_field=False):
    if near_field > 1.0:
        # Damping reduction factor
        return (0.07 / (0.02 + xi)) ** 0.25
    else:
        # Damping reduction factor
        return (0.07 / (0.02 + xi)) ** 0.5


def damping_from_reduction_factor(eta):
    """
    Inversion of reduction factor equation
    :param eta:
    :return:
    """
    return (0.07 / eta ** 2) - 0.02


def equivalent_viscous_damping(mu, mtype="concrete", btype="frame"):
    """
    Calculate the equivalent viscous damping based on the ductility and structural type.
    :param mu: Displacement ductility
    :param mtype: material type
    :param btype: building type (e.g. frame or wall)
    :return:
    """
    pie = 3.141
    if mu < 1:
        return 0.05
    if mtype == "concrete":
        if btype == "frame":
            # Equivalent viscous damping for concrete frame
            return 0.05 + 0.565 * (mu - 1) / (mu * pie)
        if btype == "wall":
            # Equivalent viscous damping for concrete wall (Sullivan et al., 2010)
            return 0.05 + 0.444 * (mu - 1) / (mu * pie)


def effective_period(delta_d, eta, corner_disp, corner_period):
    corner_disp_eff = corner_disp * eta
    if delta_d > corner_disp_eff:
        return 0.0
    else:
        return corner_period * delta_d / corner_disp_eff


def effective_period_from_stiffness(mass_eff, k_eff):
    """
    Calculates the effective period based on the mass and stiffness
    :param mass_eff: effective mass
    :param k_eff: effective stiffness
    :return:
    """
    return 2 * 3.141 * np.sqrt(mass_eff / k_eff)


def effective_stiffness_from_base_shear(v_base, disp):
    """
    Calculates the effective stiffness based on the base shear and displacement.
    Typically used in displacement based assessment
    :return:
    """
    return v_base / disp


def displacement_from_effective_period(eta, corner_disp, t_eff, corner_period):
    """
    Displacement of SDOF using displacement-based assessment. # Eq. 11 Millen et al. (2016)
    :param eta: Displacement reduction factor
    :param corner_disp: Corner spectral displacement
    :param t_eff: Effective displacement
    :param corner_period:
    :return:
    """
    if t_eff > corner_period:
        return eta * corner_disp
    return eta * corner_disp * t_eff / corner_period


def effective_stiffness(mass_eff, t_eff):
    """
    Calculates the effective period based on the mass and period
    :param mass_eff: effective mass
    :param t_eff: effective period
    :return:
    """
    return (4 * 3.141 ** 2 * mass_eff) / t_eff ** 2


def design_base_shear(k_eff, delta_d):

    return k_eff * delta_d


def bilinear_load_factor(ductility_current, ductility_max, r):
    """
    Computes the load
    :param ductility_current: Current ductility
    :param ductility_max: Maximum ductility
    :param r: post-yield bi-linear
    :return: factor to reduce maximum load
    """
    hardening_load = r * (ductility_max - 1)
    if ductility_current > ductility_max:
        raise DesignError("Current ductility: {0}, exceeds maximum ductility {1}".format(ductility_current,
                                                                                         ductility_max))
    elif ductility_current > 1.0:
        return 1.0 - hardening_load + r * (ductility_current - 1)
    else:  # Elastic behaviour
        return (1.0 - hardening_load) * ductility_current


def calculate_storey_forces(masses, displacements, v_base):
    mass_x_disp = np.array(masses) * np.array(displacements)
    storey_forces = 0.9 * v_base * mass_x_disp / sum(mass_x_disp)  # Newtons per storey
    storey_forces[-1] = storey_forces[-1] + 0.1 * v_base
    return storey_forces


def p_delta_moment(mass_eff, delta_d):
    return mass_eff * 9.8 * delta_d


def p_delta_base_shear(mass_eff, delta_d, h_eff, v_base):

    moment_pd = p_delta_moment(mass_eff, delta_d)
    pd_factor = moment_pd / v_base * h_eff
    if pd_factor > 0.1:
        c_p_delta = 0.5
        return c_p_delta * moment_pd / h_eff
    else:
        return 0


def yield_curvature(epsilon_y, length, btype="wall"):
    if btype == "wall":
        return 2.0 * epsilon_y / length


def add_foundation(ss_heights, ss_masses, fd_height, fd_mass):
    # add foundation to heights
    heights = list(ss_heights)
    heights.insert(0, 0)
    heights = np.array(heights) + fd_height
    # add foundation to masses
    storey_masses = list(ss_masses)
    storey_masses.insert(0, fd_mass)
    storey_masses = np.array(storey_masses)
    return heights, storey_masses





import numpy as np

from sfsimodels import loader as ml
from sfsimodels import output as mo

from eqdes import models as sm
from eqdes import ddbd_tools as dt
from eqdes import nonlinear_foundation as nf
from eqdes.extensions.exceptions import DesignError


def dbd_frame(fb, hz, design_drift=0.02, **kwargs):

    df = sm.DesignedFrame(fb, hz)
    df.design_drift = design_drift
    verbose = kwargs.get('verbose', df.verbose)

    for i in range(100):
        mu_reduction_factor = 1.0 - float(i) / 100
        theta_c = df.design_drift * mu_reduction_factor
        displacements = dt.displacement_profile_frame(theta_c, df.heights, df.hm_factor)
        df.delta_d, df.mass_eff, df.height_eff = dt.equivalent_sdof(df.storey_mass_p_frame, displacements, df.heights)
        df.theta_y = dt.conc_frame_yield_drift(df.fye, df.concrete.youngs_steel, df.av_bay, df.av_beam)
        delta_y = dt.yield_displacement(df.theta_y, df.height_eff)
        df.mu = dt.ductility(df.delta_d, delta_y)
        df.xi = dt.equivalent_viscous_damping(df.mu, mtype="concrete", btype="frame")
        df.eta = dt.reduction_factor(df.xi)
        df.t_eff = dt.effective_period(df.delta_d, df.eta, df.hz.corner_disp, df.hz.corner_period)

        if verbose > 1:
            print('Delta_D: ', df.delta_d)
            print('Effective mass: ', df.mass_eff)
            print('Effective height: ', df.height_eff)
            print('Mu: ', df.mu)
            print('theta yield', df.theta_y)
            print('xi: ', df.xi)
            print('Reduction Factor: ', df.eta)
            print('t_eff', df.t_eff)

        if df.t_eff > 0:
            break
        else:
            if verbose > 1:
                print("drift %.2f is not compatible" % theta_c)
    k_eff = dt.effective_stiffness(df.mass_eff, df.t_eff)
    df.v_base = dt.design_base_shear(k_eff, df.delta_d)
    df.storey_forces = dt.calculate_storey_forces(df.storey_mass_p_frame, displacements, df.v_base)
    return df


def wall(wb, hz, design_drift=0.025, **kwargs):
    """
    Displacement-based design of a concrete wall.
    :param wb: WallBuilding object
    :param hz: Hazard Object
    :param design_drift: Design drift
    :param kwargs:
    :return: DesignedWall object
    """

    dw = sm.DesignedWall(wb, hz)
    dw.design_drift = design_drift
    verbose = kwargs.get('verbose', dw.verbose)
    dw.static_dbd_values()
    # k = min(0.2 * (fu / fye - 1), 0.08)  # Eq 4.31b
    k = min(0.15 * (dw.fu / dw.fye - 1), 0.06)  # Eq 6.5a from DDBD code
    l_c = dw.max_height
    long_db = dw.preferred_bar_diameter
    l_sp = 0.022 * dw.fye * long_db / 1.0e6  # Eq 4.30
    l_p = max(k * l_c + l_sp + 0.1 * dw.wall_depth, 2 * l_sp)
    phi_y = dt.yield_curvature(dw.epsilon_y, dw.wall_depth, btype="wall")
    delta_y = dt.yield_displacement_wall(phi_y, dw.heights, dw.max_height)
    phi_p = dw.phi_material - phi_y
    # determine whether code limit or material strain governs
    theta_ss_code = design_drift
    dw.theta_y = dw.epsilon_y * dw.max_height / dw.wall_depth
    theta_p_code = (theta_ss_code - dw.theta_y)
    theta_p = min(theta_p_code, phi_p * l_p)

    # Assume no torsional effects
    increments = theta_p + dw.theta_y
    for i in range(20):
        reduced_theta_p = theta_p - increments * float(i) / 20
        if reduced_theta_p > 0.0:
            non_linear = 1

            delta_p = reduced_theta_p * (dw.heights - (0.5 * l_p - l_sp))
            if verbose > 2:
                print('reduced_theta_p: ', reduced_theta_p)
                print('delta_p: ', delta_p)

            delta_st = delta_y + delta_p
        else:
            raise DesignError('can not handle linear design, resize footing')

        dw.design_drift = reduced_theta_p + dw.theta_y

        delta_ls = delta_st

        displacements = delta_ls * dw.hm_factor

        dw.delta_d, dw.mass_eff, dw.height_eff = dt.equivalent_sdof(dw.storey_mass_p_wall, displacements, dw.heights)
        delta_y = dt.yield_displacement_wall(phi_y, dw.height_eff, dw.max_height)
        dw.mu = dt.ductility(dw.delta_d, delta_y)
        dw.xi = dt.equivalent_viscous_damping(dw.mu, mtype="concrete", btype="wall")
        dw.eta = dt.reduction_factor(dw.xi)
        dw.t_eff = dt.effective_period(dw.delta_d, dw.eta, dw.hz.corner_disp, dw.hz.corner_period)

        if verbose > 1:
            print('Delta_D: ', dw.delta_d)
            print('Effective mass: ', dw.mass_eff)
            print('Effective height: ', dw.height_eff)
            print('Mu: ', dw.mu)
            print('theta yield', dw.theta_y)
            print('xi: ', dw.xi)
            print('Reduction Factor: ', dw.eta)
            print('t_eff', dw.t_eff)

        if dw.t_eff > 0:
            break
        else:
            if verbose > 1:
                print("drift %.2f is not compatible" % reduced_theta_p)
    k_eff = dt.effective_stiffness(dw.mass_eff, dw.t_eff)
    dw.v_base = dt.design_base_shear(k_eff, dw.delta_d)
    dw.storey_forces = dt.calculate_storey_forces(dw.storey_mass_p_wall, displacements, dw.v_base)
    return dw


def dbd_sfsi_frame(fb, hz, sl, fd, deisgn_drift=0.02, found_rot=0.00001, found_rot_tol=0.02, found_rot_iterations=20, **kwargs):

    df = sm.DesignedSFSIFrame(fb, hz, sl, fd)  # TODO: change sp to sl
    df.design_drift = deisgn_drift
    df.theta_f = found_rot
    verbose = kwargs.get('verbose', df.verbose)
    df.static_values()

    # add foundation to heights and masses
    heights, storey_masses = dt.add_foundation(df.heights, df.storey_masses, df.fd.height, df.fd.mass)
    df.storey_mass_p_frame = storey_masses / df.n_seismic_frames

    # initial estimate of foundation shear
    for iteration in range(found_rot_iterations):  # iterate the foundation rotation
        df.delta_fshear = 0

        for i in range(100):
            mu_reduction_factor = 1.0 - float(i) / 100
            theta_c = df.design_drift * mu_reduction_factor
            displacements = dt.displacement_profile_frame(theta_c, heights, df.hm_factor, foundation=True,
                                                    fd_height=df.fd.height, theta_f=df.theta_f)
            df.delta_d, df.mass_eff, df.height_eff = dt.equivalent_sdof(df.storey_mass_p_frame, displacements, heights)
            df.theta_y = dt.conc_frame_yield_drift(df.fye, df.concrete.youngs_steel, df.av_bay, df.av_beam)
            delta_y = dt.yield_displacement(df.theta_y, df.height_eff - df.fd.height)

            df.delta_frot = df.theta_f * df.height_eff
            df.delta_f = df.delta_frot + df.delta_fshear
            df.delta_ss = df.delta_d - df.delta_f

            df.mu = dt.ductility(df.delta_ss, delta_y)
            if df.mu < 0:
                    raise DesignError('foundation rotation to large, Mu < 0.0')
            eta_fshear = nf.foundation_shear_reduction_factor()
            xi_ss = dt.equivalent_viscous_damping(df.mu)
            eta_ss = dt.reduction_factor(xi_ss)

            norm_rot = found_rot / df.theta_pseudo_up
            bhr = (df.fd.width / df.height_eff)
            cor_norm_rot = nf.calculate_corrected_normalised_rotation(norm_rot, bhr)
            eta_frot = nf.foundation_rotation_reduction_factor(cor_norm_rot)
            if verbose >= 1:
                print("mu: ", df.mu)
                print('DRF_ss: ', eta_ss)
                print('DRF_frot: ', eta_frot)

            eta_sys = nf.system_reduction_factor(df.delta_ss, df.delta_frot, df.delta_fshear, eta_ss, eta_frot, eta_fshear)

            df.eta = eta_sys
            df.xi = dt.damping_from_reduction_factor(eta_sys)
            df.t_eff = dt.effective_period(df.delta_d, df.eta, df.hz.corner_disp, df.hz.corner_period)

            if verbose > 1:
                print('Delta_D: ', df.delta_d)
                print('Effective mass: ', df.mass_eff)
                print('Effective height: ', df.height_eff)
                print('Mu: ', df.mu)
                print('theta yield', df.theta_y)
                print('xi: ', df.xi)
                print('Reduction Factor: ', df.eta)
                print('t_eff', df.t_eff)

            if df.t_eff > 0:
                break
            else:
                if verbose > 1:
                    print("drift %.2f is not compatible" % theta_c)
        k_eff = dt.effective_stiffness(df.mass_eff, df.t_eff)
        v_base_dynamic = dt.design_base_shear(k_eff, df.delta_d)
        v_base_p_delta = dt.p_delta_base_shear(df.mass_eff, df.delta_d, df.height_eff, v_base_dynamic)
        df.v_base = v_base_dynamic + v_base_p_delta
        df.storey_forces = dt.calculate_storey_forces(df.storey_mass_p_frame, displacements, df.v_base)

        stiffness_ratio = nf.foundation_rotation_stiffness_ratio(cor_norm_rot)
        k_f_eff = df.k_f_0 * stiffness_ratio
        temp_found_rot = found_rot
        moment_f = df.v_base * df.height_eff
        found_rot = moment_f / k_f_eff
        df.delta_fshear = df.v_base / (0.5 * df.k_f0_shear)

        iteration_diff = (abs(temp_found_rot - found_rot) / found_rot)
        if iteration_diff < found_rot_tol:
            if verbose:
                print('found_rot_i-1: ', temp_found_rot)
                print('found_rot_i: ', found_rot)
                print('n_iterations: ', iteration)
            break
        elif iteration == found_rot_iterations - 1:
            print('found_rot_i-1: ', temp_found_rot)
            print('found_rot_i: ', found_rot)
            print('iteration difference: ', iteration_diff)
            df.mu = -1
            raise DesignError('Could not find convergence')


        df.theta_f = found_rot
    return df


def run_frame_fixed():
    fb = sm.FrameBuilding()
    hz = sm.Hazard()
    hz = ml.load_hazard_sample_data(hz)
    ml.load_frame_building_sample_data(fb)
    designed_frame = dbd_frame(fb, hz)
    print(designed_frame.n_seismic_frames)
    para = [mo.output_to_table(hz)]
    para.append(mo.output_to_table(fb))
    para.append(mo.add_table_ends(para))
    print("".join(para))

    for item in hz.__dict__:
        print(item, hz.__dict__[item])





def run_frame_sfsi():
    fb = sm.FrameBuilding()
    hz = sm.Hazard()
    sp = sm.Soil()
    fd = sm.RaftFoundation()
    ml.load_hazard_sample_data(hz)
    ml.load_frame_building_sample_data(fb)
    ml.load_soil_sample_data(sp)
    ml.load_foundation_sample_data(fd)
    designed_frame = dbd_sfsi_frame(fb, hz, sp, fd, verbose=0)
    para = mo.output_to_table(designed_frame, olist="all")
    para += mo.output_to_table(fd)
    para += mo.output_to_table(sp)
    para += mo.output_to_table(hz)

    para = mo.add_table_ends(para)
    print(para)


if __name__ == '__main__':
    # run_frame_sfsi()
    # run_frame_dba_fixed()
    run_frame_fixed()



import numpy as np
from eqdes.extensions.exceptions import DesignError


def foundation_rotation_reduction_factor_millen(cor_norm_rot):
    """
    Returns the displacement reduction factor of the foundation for a given corrected normalised rotation.

    :param cor_norm_rot:
    """

    dmf = np.sqrt(1.0 / (1.0 + 5.0 * (1 - np.exp(-0.15 * cor_norm_rot))))
    return dmf


def foundation_rotation_reduction_factor(cor_norm_rot, method="Millen"):
    """
    Returns the displacement reduction factor of the foundation for a given rotation
    :param cor_norm_rot:
    """

    func_map = {
        "Millen": foundation_rotation_reduction_factor_millen,

    }
    return func_map[method](cor_norm_rot)


def foundation_shear_reduction_factor():
    """
    Returns the displacement reduction factor for soil-foundation shear deformation.
    :param cor_norm_rot:
    """
    # According to Millen (2016)
    return 0.76


def foundation_stiffness_ratio_paolucci(ALR, FR, DR):
    """
    From Paolucci et al. (2013)
    Returns the degradation of foundation stiffness for a given foundation rotation
    :param ALR: axial load ratio
    :param FR: foundation rotation
    :param DR: soil relative density
    """
    # TODO: refactor into a dictionary
    if DR == 90:
        Nratio = [2, 3, 4.5, 6, 7.5, 9, 10, 15, 20, 25, 30]
        a = [
            458.36,
            281.95,
            262.81,
            292.81,
            324.76,
            378.05,
            415.5,
            575.36,
            1010.99,
            2461.06,
            5192.13]
        m = [1.30, 1.11, 1.00, 0.94, 0.91, 0.89, 0.88, 0.83, 0.86, 0.95, 1.02]
    elif DR == 60:
        Nratio = [2, 3, 4.5, 6, 7.5, 9, 10, 15, 20, 25, 30]
        a = [
            686.26,
            386.24,
            339.87,
            352.13,
            398.44,
            433.12,
            452.44,
            653.02,
            1219.47,
            2461.06,
            5192.13]
        m = [1.30, 1.11, 0.98, 0.92, 0.89, 0.86, 0.84, 0.79, 0.83, 0.89, 0.96]

    for A in range(len(Nratio) - 1):
        if ALR <= Nratio[A + 1] and ALR >= Nratio[A]:
            stiffvals = np.zeros((2))
            stiffvals[0] = 1.0 / (1.0 + a[A] * FR ** m[A])
            stiffvals[1] = 1.0 / (1.0 + a[A + 1] * FR ** m[A + 1])
            stiffRatio = np.interp(ALR, [Nratio[A], Nratio[A + 1]], stiffvals)

    return stiffRatio


def foundation_damping_paolucci(ALR, foundation_rotation, DR):
    """
    Returns the damping of the foundation for a given rotation
    :param ALR: Axial load ratio
    :param foundation_rotation:
    :param DR:
    """
    # TODO: refactor into a dictionary
    zeta_min = 0.036
    if DR == 90:
        Nratio = [2, 3, 4.5, 6, 7.5, 9, 10, 15, 20, 25, 30]
        alpha = [
            27.73,
            32.76,
            43.93,
            62.25,
            66.96,
            85.08,
            95.60,
            164.42,
            233.70,
            305.97,
            382.51]  # 4th number a bit high
        zeta_max = 0.25
    elif DR == 60:
        Nratio = [2, 3, 4.5, 6, 7.5, 9, 10, 15, 20, 25, 30]
        alpha = [
            39.39,
            47.61,
            67.79,
            90.64,
            104.49,
            119.20,
            130.85,
            210.42,
            285.15,
            367.70,
            442.47]
        zeta_max = 0.37

    for A in range(len(Nratio) - 1):
        if ALR <= Nratio[A + 1] and ALR >= Nratio[A]:
            dmpingvalues = np.zeros((2))
            dmpingvalues[0] = (zeta_min + (zeta_max - zeta_min) *
                (1 - np.exp(-alpha[A] * foundation_rotation)))
            dmpingvalues[1] = (zeta_min + (zeta_max - zeta_min) *
                (1 - np.exp(-alpha[A + 1] * foundation_rotation)))
            damping = np.interp(ALR, [Nratio[A], Nratio[A + 1]], dmpingvalues)

    return damping


def foundation_rotation_stiffness_ratio_millen(cor_norm_rot):
    """
    From Millen Thesis (2016)
    Returns the degradation of foundation stiffness for a given normalised foundation rotation.
    """
    if hasattr(cor_norm_rot, "__len__"):
        stiff_values = []
        for cnr in cor_norm_rot:
            stiff_values.append(foundation_rotation_stiffness_ratio_millen(cnr))
        return np.array(stiff_values)

    inter = 0.8
    slope = -0.04
    if cor_norm_rot > 100:
        raise DesignError('cor_norm_rot exceeds equation limits. %.3f <= 100' % cor_norm_rot)
    stiff_ratio = min(1.0, -0.7 * (1 - np.exp(-0.18 * cor_norm_rot)) + inter + slope * np.log10(cor_norm_rot))

    return stiff_ratio


def foundation_rotation_stiffness_ratio(cor_norm_rot, method="Millen"):
    """
    Returns the degradation of foundation stiffness for a given normalised foundation rotation
    """
    func_map = {
        "Millen": foundation_rotation_stiffness_ratio_millen,

    }
    return func_map[method](cor_norm_rot)


def calculate_pseudo_uplift_angle(weight, width, k_f_0, axial_load_ratio, alpha, zeta=1.5):
    """
    Calculates the pseudo uplift angle according to Chatzigogos et al. (2011)
    :param weight:
    :param width:
    :param k_f_0:
    :param axial_load_ratio:
    :param alpha:
    :param zeta:
    :return:
    """
    return (weight * width / alpha / k_f_0) * np.exp(-zeta * 1.0 / axial_load_ratio)


def calculate_corrected_normalised_rotation(norm_rot, bhr):
    """
    Corrected normalised rotation according to Millen (2015)
    Correction is for shear force
    Normalisation is against pseudo uplift angle
    :param norm_rot: Normalised rotation angle
    :param bhr: Ratio of Foundation width to Effective height of superstructure mass
    :return:
    """
    return norm_rot ** (1 - 0.2 * bhr) * 10 ** (.25 * bhr)


def system_reduction_factor(delta_ss, delta_frot, delta_fshear, eta_ss, eta_frot, eta_fshear):
    """
    Calculates the system displacement reduction factor based on the foundation and superstrucutre
    displacement reduction factors.
    :param delta_ss: superstructure displacement
    :param delta_frot: displacement due to foundation rotation
    :param delta_fshear: displacement due to soil-foundation shear deformation
    :param eta_ss: superstructure displacement reduction factor
    :param eta_frot: foundation rotation displacement reduction factor
    :param eta_fshear: soil foundation shear deformation displacement reduction factor
    :return:
    """
    delta_total = delta_ss + delta_frot + delta_fshear
    return (delta_ss * eta_ss + delta_frot * eta_frot + delta_fshear * eta_fshear) / delta_total


def bearing_capacity(f_area, soil_q):
    return f_area * soil_q







import numpy as np

from sfsimodels import models as sm
from sfsimodels import output as mo
import geofound
from eqdes import nonlinear_foundation as nf
from eqdes import ddbd_tools as dt
from eqdes.extensions.exceptions import DesignError

class Soil(sm.Soil):
    required_inputs = ["g_mod",
                      "phi",
                      "unit_weight"
                      ]


class Hazard(sm.SeismicHazard):
    required_inputs = ["corner_disp",
                      "corner_period",
                      "z_factor",
                      "r_factor"
                      ]


class FrameBuilding(sm.FrameBuilding):
    required_inputs = ["interstorey_heights",
                      "floor_length",
                      "floor_width",
                      "storey_masses",
                      "bay_lengths",
                      "beam_depths",
                      "n_seismic_frames",
                      "n_gravity_frames"
                      ]

    def to_table(self, table_name="fb-table"):
        para = mo.output_to_table(self, olist="all")
        para += mo.output_to_table(self.hz)
        para = mo.add_table_ends(para, 'latex', table_name, table_name)
        return para


class WallBuilding(sm.WallBuilding):
    required_inputs = [
        'floor_length',
        'floor_width',
        'interstorey_heights',
        'n_storeys',
        "n_walls",
        "wall_depth"
    ]

    def to_table(self, table_name="wb-table"):
        para = mo.output_to_table(self, olist="all")
        para = mo.add_table_ends(para, 'latex', table_name, table_name)
        return para


class Concrete(sm.material.Concrete):
    required_inputs = [
            'fy',
            'youngs_steel'
    ]


class RaftFoundation(sm.RaftFoundation):
    required_inputs = [
        "width",
        "length",
        "depth",
        "height",
        "density",
        "i_ww",
        "i_ll"
    ]


class PadFoundation(sm.PadFoundation):
    required_inputs = [
        "width",
        "length",
        "depth",
        "height",
        "density",
        "i_ww",
        "i_ll",
        "n_pads_l",
        "n_pads_w",
        "pad_length",
        "pad_width",
    ]


class DesignedFrame(FrameBuilding):
    method = "standard"

    hz = Hazard()

    # outputs
    design_drift = 0.0
    delta_d = 0.0
    mass_eff = 0.0
    height_eff = 0.0
    theta_y = 0.0
    mu = 0.0
    xi = 0.0
    eta = 0.0
    t_eff = 0.0
    v_base = 0.0
    storey_forces = 0.0

    def __init__(self, fb, hz, verbose=0):
        super(DesignedFrame, self).__init__()  # run parent class initialiser function
        self.__dict__.update(fb.__dict__)
        self.hz.__dict__.update(hz.__dict__)
        self.verbose = verbose
        self.av_beam = np.average(self.beam_depths)
        self.av_bay = np.average(self.bay_lengths)
        self.fye = 1.1 * self.concrete.fy
        self.storey_mass_p_frame = self.storey_masses / self.n_seismic_frames
        self.storey_forces = np.zeros((1, len(self.storey_masses)))
        self.hm_factor = dt.cal_higher_mode_factor(self.n_storeys, btype="frame")

    @property
    def inputs(self):
        input_list = super(DesignedFrame, self).inputs
        new_inputs = [
        "method",
    ]
        return input_list + new_inputs


class DesignedWall(WallBuilding):
    method = "standard"
    preferred_bar_diameter = 0.032

    hz = Hazard()

    # outputs
    phi_material = 0.0
    epsilon_y = 0.0
    fu = 0.0
    design_drift = 0.0
    delta_d = 0.0
    mass_eff = 0.0
    height_eff = 0.0
    theta_y = 0.0
    mu = 0.0
    xi = 0.0
    eta = 0.0
    t_eff = 0.0
    v_base = 0.0
    storey_forces = 0.0

    def __init__(self, wb, hz, verbose=0):
        super(DesignedWall, self).__init__()  # run parent class initialiser function
        self.__dict__.update(wb.__dict__)
        self.hz.__dict__.update(hz.__dict__)
        self.verbose = verbose
        self.fye = 1.1 * self.concrete.fy
        self.storey_mass_p_wall = self.storey_masses / self.n_walls
        self.storey_forces = np.zeros((1, len(self.storey_masses)))
        self.hm_factor = dt.cal_higher_mode_factor(self.n_storeys, btype="wall")

    @property
    def inputs(self):
        input_list = super(DesignedWall, self).inputs
        new_inputs = [
        "method",
    ]
        return input_list + new_inputs

    def static_dbd_values(self):
        # Material strain limits check
        self.phi_material = 0.072 / self.wall_depth  # Eq 6.10b
        self.fye = 1.1 * self.concrete.fy
        self.epsilon_y = self.fye / self.concrete.youngs_steel
        self.fu = 1.40 * self.fye  # Assumed, see pg 141


class AssessedFrame(FrameBuilding):
    method = "standard"
    post_yield_stiffness_ratio = 0.05

    hz = Hazard()

    # inputs
    otm_max = 0.0
    theta_max = 0.0

    # outputs
    assessed_drift = 0.0
    delta_max = 0.0
    mass_eff = 0.0
    height_eff = 0.0
    theta_y = 0.0
    max_mu = 0.0
    mu = 0.0
    xi = 0.0
    eta = 0.0
    t_eff = 0.0
    v_base = 0.0
    storey_forces = 0.0

    def __init__(self, fb, hz, verbose=0):
        super(AssessedFrame, self).__init__()  # run parent class initialiser function
        self.__dict__.update(fb.__dict__)
        self.hz.__dict__.update(hz.__dict__)
        self.verbose = verbose
        self.av_beam = np.average(self.beam_depths)
        self.av_bay = np.average(self.bay_lengths)
        self.fye = 1.1 * self.concrete.fy
        self.storey_mass_p_frame = self.storey_masses / self.n_seismic_frames
        self.storey_forces = np.zeros((1, len(self.storey_masses)))
        self.hm_factor = dt.cal_higher_mode_factor(self.n_storeys, btype="frame")

    @property
    def inputs(self):
        input_list = super(AssessedFrame, self).inputs
        new_inputs = [
        "method",
    ]
        return input_list + new_inputs


class DesignedSFSIFrame(DesignedFrame):

    sl = sm.Soil()
    fd = sm.RaftFoundation()
    total_weight = 0.0
    theta_f = 0.0
    axial_load_ratio = 0.0
    theta_pseudo_up = 0.0

    def __init__(self, fb, hz, sl, fd):
        super(DesignedSFSIFrame, self).__init__(fb, hz)  # run parent class initialiser function
        self.sl.__dict__.update(sl.__dict__)
        self.fd.__dict__.update(fd.__dict__)
        self.k_f0_shear = geofound.shear_stiffness(self.fd.width, self.fd.length, self.sl.g_mod, self.sl.poissons_ratio)

        if self.fd.ftype == "raft":
            self.k_f_0 = geofound.rotational_stiffness(self.sl, self.fd)
            self.alpha = 4.0
        else:
            self.k_f_0 = geofound.rotational_stiffness(self.sl, self.fd)
            self.alpha = 3.0

        self.zeta = 1.5

    def static_values(self):
        self.total_weight = (sum(self.storey_masses) + self.fd.mass) * self.g
        soil_q = geofound.capacity_salgado_2008(sl=self.sl, fd=self.fd)

        # Deal with both raft and pad foundations
        bearing_capacity = nf.bearing_capacity(self.fd.area, soil_q)
        weight_per_frame = sum(self.storey_masses) / (self.n_seismic_frames + self.n_gravity_frames) * self.g
        self.axial_load_ratio = bearing_capacity / self.total_weight

        self.theta_pseudo_up = nf.calculate_pseudo_uplift_angle(self.total_weight, self.fd.width, self.k_f_0,
                                                                self.axial_load_ratio, self.alpha, self.zeta)

    def to_table(self, table_name="df-table"):
        para = mo.output_to_table(self, olist="all")
        para += mo.output_to_table(self.fd)
        para += mo.output_to_table(self.sl)
        para += mo.output_to_table(self.hz)
        para = mo.add_table_ends(para,'latex', table_name, table_name)
        return para


class AssessedSFSIFrame(AssessedFrame):
    sl = sm.Soil()
    fd = sm.Foundation()
    total_weight = 0.0
    theta_f = 0.0
    axial_load_ratio = 0.0
    theta_pseudo_up = 0.0

    def __init__(self, fb, hz, sl, fd):
        super(AssessedSFSIFrame, self).__init__(fb, hz)  # run parent class initialiser function
        self.sl.__dict__.update(sl.__dict__)
        if fd.ftype == "raft":
            self.fd = sm.RaftFoundation()
        if fd.ftype == "pad":
            self.fd = sm.PadFoundation()
        self.fd.__dict__.update(fd.__dict__)
        self.k_f0_shear = geofound.shear_stiffness(self.fd.width, self.fd.length, self.sl.g_mod, self.sl.poissons_ratio)

        if self.fd.ftype == "raft":
            self.k_f_0 = geofound.rotational_stiffness(self.sl, self.fd)
            #self.k_f_0 = nf.rotational_stiffness(self.fd.width, self.fd.length, self.sl.g_mod, self.sl.poissons_ratio)
            self.alpha = 4.0
        else:
            self.k_f_0 = geofound.rotational_stiffness(self.sl, self.fd)
            #self.k_f_0 = nf.rotational_stiffness(self.fd.width, self.fd.length, self.sl.g_mod, self.sl.poissons_ratio) / 2
            self.alpha = 3.0

        self.zeta = 1.5

    def static_values(self):
        self.total_weight = (sum(self.storey_masses) + self.fd.mass) * self.g
        self.soil_q = geofound.capacity_salgado_2008(sl=self.sl, fd=self.fd)
        # Add new function to foundations, bearing_capacity_from_sfsimodels,
        # Deal with both raft and pad foundations
        bearing_capacity = nf.bearing_capacity(self.fd.area, self.soil_q)
        weight_per_frame = sum(self.storey_masses) / (self.n_seismic_frames + self.n_gravity_frames) * self.g
        self.axial_load_ratio = bearing_capacity / self.total_weight
        if self.axial_load_ratio < 1.0:
            raise DesignError("Static failure expected. Axial load ratio: %.3f" % self.axial_load_ratio)

        self.theta_pseudo_up = nf.calculate_pseudo_uplift_angle(self.total_weight, self.fd.width, self.k_f_0,
                                                                self.axial_load_ratio, self.alpha, self.zeta)

    def to_table(self, table_name="af-table"):
        para = mo.output_to_table(self, olist="all")
        para += mo.output_to_table(self.fd, prefix="Foundation ")
        para += mo.output_to_table(self.sl, prefix="Soil ")
        para += mo.output_to_table(self.hz, prefix="Hazard ")
        para = mo.add_table_ends(para, 'latex', table_name, table_name)
        return para
